% Log-barrier constraint function
%
% constraints is a cell array of scalar constraint functions
% xx is the current iterate
%
% value is the function value at xx
% gradient is the gradient at xx
% hessian is the hessian at xx
%
function [ value, gradient, hessian ] = objective_scalar_constraints( constraints, xx )

	if ( length( constraints ) < 1 ); error( 'constraint cell array cannot be empty' ); end

	% we need to make sure that we get infinity (not a complex number) for xx which
	% violate the constraints
	function ll = safelog( pp )
		ll = log( max( 0, pp ) );
	end


	values = {};
	gradients = {};
	hessians = {};

	% fill in values{}, gradients{} and hessians{} with the values, gradients and
	% hessians of each constraint
	for ii = 1:length( constraints )

		% find the value, matrix derivatives, and matrix second derivatives of the
		% constraint
		if ( nargout < 2 )
			[ constraint_value ] = constraints{ ii }( xx );
		elseif ( nargout < 3 )
			[ constraint_value, constraint_gradient ] = constraints{ ii }( xx );
		else
			[ constraint_value, constraint_gradient, constraint_hessian ] = constraints{ ii }( xx );
		end
		if ( length( constraint_value ) > 1 ); error( 'objective_scalar_constraints cannot handle matrix constraints' ); end

		value = -sum( safelog( eig( constraint_value ) ) );
		values = [ values, { value } ];

		if ( nargout > 1 )

			%gradient = ( TODO: gradient of -logdet( constraint ), evaluated at xx );
			%    NOTE: constraint_value is the value of the constraint function at xx
			%    NOTE: constraint_gradient is the gradient of the constraint function at xx
			gradients = [ gradients, { gradient } ];

			if ( nargout > 2 )

				%hessian = ( TODO: Hessian of -logdet( constraint ), evaluated at xx );
				%    NOTE: constraint_value is the value of the constraint function at xx
				%    NOTE: constraint_gradient is the gradient of the constraint function at xx
				%    NOTE: constraint_hessian is the Hessian of the constraint function at xx
				hessians = [ hessians, { hessian } ];

			end
		end

	end


	% sum the values for all constraints
	value = values{ 1 };
	for ii = 2:length( values )
		value = value + values{ ii };
	end

	if ( nargout > 1 )

		% sum the gradients for all constraints
		gradient = gradients{ 1 };
		for ii = 2:length( gradients )
			gradient = gradient + gradients{ ii };
		end

		if ( nargout > 2 )

			% sum the hessians for all constraints
			hessian = hessians{ 1 };
			for ii = 2:length( hessians )
				hessian = hessian + hessians{ ii };
			end
		end
	end

end
